/*
 ***********************************************************************************************
 *                                                                                              
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 
 *                                                                                              
 *  OSA is distributed under BSD license (see license.txt)                                      
 *                                                                                              
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro                                  
 *              http://picosa.narod.ru                                                          
 *                                                                                              
 *----------------------------------------------------------------------------------------------
 *                                                                                              
 *  File:       osa_csem.h                                                                      
 *                                                                                              
 *  Programmer: Timofeev Victor                                                                 
 *              osa@pic24.ru, testerplus@mail.ru                                                
 *                                                                                              
 *  Definition: Services for work with counting semaphores                                      
 *                                                                                              
 *  History:    21.01.2009                                                                      
 *              21.03.2009 -    Enable/disable interrupt actions on SignalCsem are placed incide
 *                              macro OS_Csem_Signal and deleted from _OS_Csem_Signal           
 *                                                                                              
 *              10.12.2009 -    1. Services for interrupt were corrected.
 *                              2. Service OS_Csem_Signal waits for semaphore is free.
 *                                 Service OS_Csem_Signal_Now added
 *                                                                                              
 ************************************************************************************************
 */


/************************************************************************************************
 *                                                                                              *
 *                            C O U N T I N G   S E M A P H O R E S                             *
 *                                                                                              *
 ************************************************************************************************/




#ifdef OS_ENABLE_CSEM


// These services are used in countsem functions (see OSA.C)

#if defined(OS_ENABLE_INT_CSEM) && (OS_CSEM_SIZE > 1)

    #define __OS_CSEM_DI()      OS_DI_INT() 
    #define __OS_CSEM_RI()      OS_RI_INT() 

#else

    #define __OS_CSEM_DI()
    #define __OS_CSEM_RI()

#endif



// Increase counting semaphore's value by 1

#define OS_Csem_Signal(countsem)                                        \
    OSM_BEGIN {                                                         \
        for (;;) {                                                      \
            __OS_CSEM_DI();                                             \
            _OS_Csem_Signal(&(countsem));                               \
            __OS_CSEM_RI();                                             \
            if (!OS_IsEventError()) break;                              \
            OS_Yield();                                                 \
        }                                                               \
    } OSM_END



// Increase counting semaphore's value by 1

#define OS_Csem_Signal_Now(countsem)                       /*91210*/    \
    OSM_BEGIN {                                                         \
        __OS_CSEM_DI();                                                 \
        _OS_Csem_Signal(&(countsem));                                   \
        __OS_CSEM_RI();                                                 \
    } OSM_END


// Set counting semaphore's value to given value

#define __OS_Csem_SetValue(countsem,value)      countsem = value

#define OS_Csem_SetValue(countsem,value)                                \
    OSM_BEGIN {                                                         \
        __OS_CSEM_DI();                                                 \
        __OS_Csem_SetValue(countsem,value);                             \
        __OS_CSEM_RI();                                                 \
    } OSM_END


// Accepting counting semaphore. Decrease it's value by 1.

#define OS_Csem_Accept(countsem)                                        \
    OSM_BEGIN {                                                         \
        __OS_CSEM_DI();                                                 \
        countsem--;                                                     \
        __OS_CSEM_RI();                                                 \
    } OSM_END


// Accepting counting semaphore (internal). Decrease it's value by 1.

#define __OS_Csem_Accept(countsem)    countsem--

// Set counting semaphore's value to 1.
#define OS_Csem_Set(countsem)    OS_Csem_SetValue(countsem, 1)

// Clear counting semaphore
#define OS_Csem_Reset(countsem)  OS_Csem_SetValue(countsem, 0)

#define OS_Csem_Create(countsem) OS_Csem_SetValue(countsem, 0)

#define __OS_Csem_Check(countsem)   (countsem)

//------------------------------------------------------------------------------
#if (OS_CSEM_SIZE == 1) || (!defined(OS_ENABLE_INT_CSEM))

    // Check counting semaphore for non zero value
    #define OS_Csem_Check(countsem)     __OS_Csem_Check(countsem)

#else

    // Check counting semaphore for non zero value

    #if defined(OS_ENABLE_INT_CSEM)
        #define OS_Csem_Check(countsem) (__OS_CSEM_DI(),_OS_temp = __OS_Csem_Check(countsem),__OS_CSEM_RI(),_OS_temp)
    #else
        #define OS_Csem_Check(countsem) __OS_Csem_Check(countsem)
    #endif

#endif
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#ifdef OS_ENABLE_INT_CSEM

    // Set counting semaphore's value to given value
    #define OS_Csem_SetValue_I(countsem,value)          countsem = value

    // Set counting semaphore's value to 1
    #define OS_Csem_Set_I(countsem)                     __OS_Csem_SetValue(countsem, 1)

    // Clear counting semaphore
    #define OS_Csem_Reset_I(countsem)                   __OS_Csem_SetValue(countsem, 0)

    // Check counting semaphore for non zero value
    #define OS_Csem_Check_I(countsem)                   __OS_Csem_Check(countsem)

    // Accept counting semaphore. Decrease it's value by 1
    #define OS_Csem_Accept_I(countsem)                  __OS_Csem_Accept(countsem)

    // Increase counting semaphore's value by 1
    #define OS_Csem_Signal_I(countsem)                  _OS_Csem_Signal_I(&countsem)    /*91210*/

#endif


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_CSEM)
//------------------------------------------------------------------------------

    // Wait counting semaphore
    #define OS_Csem_Wait(countsem)                                          \
        OSM_BEGIN {                                                         \
            for (;;) {                                                      \
                _OS_WAIT_EVENT(__OS_Csem_Check(countsem));                  \
                __OS_CSEM_DI();                                             \
                if (__OS_Csem_Check(countsem)) break;                       \
                __OS_CSEM_RI();                                             \
            }                                                               \
            __OS_Csem_Accept(countsem);                                     \
            __OS_CSEM_RI();                                                 \
        } OSM_END

    // Wait counting semaphore. Exit if timeout expired.
    #define OS_Csem_Wait_TO(countsem, timeout)                              \
        OSM_BEGIN {                                                         \
            for (;;) {                                                      \
                _OS_WAIT_EVENT_TIMEOUT(__OS_Csem_Check(countsem), timeout); \
                __OS_CSEM_DI();                                             \
                if (__OS_Csem_Check(countsem) || OS_IsTimeout()) break;     \
                __OS_CSEM_RI();                                             \
            }                                                               \
            if (!OS_IsTimeout()) {                                          \
                __OS_Csem_Accept(countsem);                                 \
            }                                                               \
            __OS_CSEM_RI();                                                 \
        } OSM_END

//------------------------------------------------------------------------------
#else
//------------------------------------------------------------------------------

    // Wait counting semaphore
    #define OS_Csem_Wait(countsem)                                          \
        OSM_BEGIN {                                                         \
            _OS_WAIT_EVENT(__OS_Csem_Check(countsem));                      \
            __OS_Csem_Accept(countsem);                                     \
        } OSM_END

    // Wait counting semaphore. Exit if timeout expired.
    #define OS_Csem_Wait_TO(countsem, timeout)                              \
        OSM_BEGIN {                                                         \
            _OS_WAIT_EVENT_TIMEOUT(__OS_Csem_Check(countsem), timeout);     \
            if (!OS_IsTimeout()) {                                          \
                __OS_Csem_Accept(countsem);                                 \
            }                                                               \
        } OSM_END

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#endif  // OS_ENABLE_CSEM



